@@ -7,7 +7,7 @@
  *
  * based on sources acquired from Plustek Inc.
  * Copyright (C) 1998 Plustek Inc.
- * Copyright (C) 2000-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2000-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  * also based on the work done by Rick Bronson <rick@efn.org>
  *.............................................................................
  * History:
@@ -736,7 +736,7 @@ typedef struct _SCANDEF
     ULong			dwScanOrigin;   /* where to start the scan              */
     Bool			fRefreshState;  /* refresh ?                            */
     Bool			fMotorBackward;
-    UChar			motorPower;     /* how to driver the motor              */
+    UChar			motorPower;     /* how to drive the motor               */
 
     ULong			dwMinReadFifo;
     ULong			dwMaxReadFifo;
