@@ -7,7 +7,7 @@
  *  @brief The interface functions to the USB driver stuff.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2002 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -25,6 +25,18 @@
  *        - made 16-bit gray mode work
  *        - added special handling for Genius devices
  *        - added TPA autodetection for EPSON Photo
+ *        - fixed bug that causes warmup each time autodetected<br>
+ *          TPA on EPSON is used
+ *        - removed Genius from PCB-Id check
+ *        - added Compaq to the list
+ *        - removed the scaler stuff for CIS devices
+ *        - removed homeing stuff from readline function
+ *        - fixed flag setting in usbDev_startScan()
+ * - 0.46 - added additional branch to support alternate calibration
+ * - 0.47 - added special handling with 0x400 vendor ID and model override
+ *        - removed PATH_MAX
+ *        - change usbDev_stopScan and usbDev_open prototype
+ *        - cleanup
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -67,71 +79,118 @@
  * <hr>
  */
 
-/**
- * to allow different vendors...
+/** useful for description tables
+ */
+typedef struct {
+	int	  id;
+	char *desc;
+} TabDef, *pTabDef;
+  
+/** to allow different vendors...
  */
 static TabDef usbVendors[] = {
 
 	{ 0x07B3, "Plustek"         },
-	{ 0x0400, "Mustek"          },  /* this in fact is not correct */
-                                    /* but used for the BearPaws   */
+	{ 0x0400, "Mustek"          },  /* this in fact is not correct  */
+                                    /* but is used for the BearPaws */
 	{ 0x0458, "KYE/Genius"      },
 	{ 0x03F0, "Hewlett-Packard" },
 	{ 0x04B8, "Epson"           },
 	{ 0x04A9, "Canon"           },
  	{ 0x1606, "UMAX"            },
-	
+ 	{ 0x049F, "Compaq"          },
 	{ 0xFFFF, NULL              }
 };
 
 /** for autodetection */
-SANE_Char USB_devname[1024];
+static SANE_Char USB_devname[1024];
 
 /** we use at least 8 megs for scanning... */
 #define _SCANBUF_SIZE (8 * 1024 * 1024)
 
 /********************** the USB scanner interface ****************************/
 
+/** remove the slash out of the model-name to obtain a valid filename
+ */
+static SANE_Bool usb_normFileName( char *fname, char* buffer, u_long max_len )
+{
+	char *src, *dst;
+	
+	if( NULL == fname )
+		return SANE_FALSE;
+		
+	if( strlen( fname ) >= max_len )
+		return SANE_FALSE;
+
+	src = fname;
+	dst = buffer;
+	while( *src != '\0' ) {
+
+		if((*src == '/') || isspace(*src))
+			*dst = '_';
+		else
+			*dst = *src;
+
+		*dst++;
+		*src++;
+	}
+	*dst = '\0';
+		
+	return SANE_TRUE;	
+}
+
 /**
  * assign the values to the structures used by the currently found scanner
  */
 static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 {
+	char     *ptr;
+	char      tmp_str1[PATH_MAX];
+	char      tmp_str2[PATH_MAX];
 	int       i;
 	ScanParam sParam;
+	u_short   tmp = 0;
 
-	DBG( _DBG_INFO, "usb_initDev(%d,0x%04x,%u)\n",
-											idx, vendor, dev->initialized );
+	DBG( _DBG_INFO, "usb_initDev(%d,0x%04x,%i)\n",
+	                idx, vendor, dev->initialized );
+	/* save capability flags... */
+	if( dev->initialized >= 0 ) {
+		tmp = DEVCAPSFLAG_TPA;
+	}
 
-	/* copy the original values... */											
+	/* copy the original values... */
 	memcpy( &dev->usbDev.Caps, Settings[idx].pDevCaps, sizeof(DCapsDef));
 	memcpy( &dev->usbDev.HwSetting, Settings[idx].pHwDef, sizeof(HWDef));
 
+	/* restore capability flags... */
+	if( dev->initialized >= 0 ) {
+		dev->usbDev.Caps.wFlags |= tmp;
+	}
+
 	if( dev->adj.warmup >= 0 )
-        dev->usbDev.dwWarmup = dev->adj.warmup;
+		dev->usbDev.dwWarmup = dev->adj.warmup;
 	
 	if( dev->adj.lampOff >= 0 )
-        dev->usbDev.dwLampOnPeriod = dev->adj.lampOff;
+		dev->usbDev.dwLampOnPeriod = dev->adj.lampOff;
 
-    if( dev->adj.lampOffOnEnd >= 0 )
-        dev->usbDev.bLampOffOnEnd = dev->adj.lampOffOnEnd;
+	if( dev->adj.lampOffOnEnd >= 0 )
+		dev->usbDev.bLampOffOnEnd = dev->adj.lampOffOnEnd;
 
-    if( dev->adj.skipCalibration > 0 )
+	if( dev->adj.skipCalibration > 0 )
 		dev->usbDev.Caps.workaroundFlag |= _WAF_BYPASS_CALIBRATION;
 
-    if( dev->adj.skipFine > 0 )
+	if( dev->adj.skipFine > 0 )
 		dev->usbDev.Caps.workaroundFlag |= _WAF_SKIP_FINE;
 
-    if( dev->adj.skipFineWhite > 0 )
+	if( dev->adj.skipFineWhite > 0 )
 		dev->usbDev.Caps.workaroundFlag |= _WAF_SKIP_WHITEFINE;
-		
-    if( dev->adj.invertNegatives > 0 )
+
+	if( dev->adj.invertNegatives > 0 )
 		dev->usbDev.Caps.workaroundFlag |= _WAF_INV_NEGATIVE_MAP;
 
 	DBG( _DBG_INFO, "Device WAF: 0x%08lx\n", dev->usbDev.Caps.workaroundFlag );
-			
-	/*
-	 * adjust data origin
+
+	/* adjust data origin
 	 */
 	dev->usbDev.Caps.Positive.DataOrigin.x -= dev->adj.tpa.x;	
 	dev->usbDev.Caps.Positive.DataOrigin.y -= dev->adj.tpa.y;	
@@ -142,9 +201,8 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	dev->usbDev.Caps.Normal.DataOrigin.x -= dev->adj.pos.x;	
 	dev->usbDev.Caps.Normal.DataOrigin.y -= dev->adj.pos.y;
 
-	/*
-     * adjust shading position
-     */
+	/** adjust shading position
+	 */
 	if( dev->adj.posShadingY >= 0 )
 		dev->usbDev.Caps.Normal.ShadingOriginY   = dev->adj.posShadingY;
 
@@ -153,47 +211,67 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 
 	if( dev->adj.negShadingY >= 0 )
 		dev->usbDev.Caps.Negative.ShadingOriginY = dev->adj.negShadingY;
-		
-	/*
-	 * the following you normally get from the registry...
+
+	/* adjust the gamma settings... */
+	if( dev->adj.rgamma == 1.0 )
+		dev->adj.rgamma = dev->usbDev.HwSetting.gamma;
+	if( dev->adj.ggamma == 1.0 )
+		dev->adj.ggamma = dev->usbDev.HwSetting.gamma;
+	if( dev->adj.bgamma == 1.0 )
+		dev->adj.bgamma = dev->usbDev.HwSetting.gamma;
+	if( dev->adj.graygamma == 1.0 )
+		dev->adj.graygamma = dev->usbDev.HwSetting.gamma;
+
+/* FIXME: not needed here??? */
+#if 0
+	if( dev->adj.rlampoff > 0 )
+		dev->usbDev.HwSetting.red_lamp_off = dev->adj.rlampoff;
+
+	if( dev->adj.glampoff > 0 )
+		dev->usbDev.HwSetting.green_lamp_off = dev->adj.glampoff;
+
+	if( dev->adj.blampoff > 0 )
+		dev->usbDev.HwSetting.blue_lamp_off = dev->adj.blampoff;
+#endif
+
+	/* the following you normally get from the registry...
 	 */
 	bMaxITA = 0;     /* Maximum integration time adjust */
-		
+
 	dev->usbDev.ModelStr = Settings[idx].pModelString;
 
 	dev->fd = handle;
 
 	/* check for TPA on EPSON device
 	 */
-	if( !dev->initialized && vendor == 0x04B8 ) {
+	if((dev->initialized < 0) && vendor == 0x04B8 ) {
 
 		u_char t;
 
-		usb_switchLampX   ( dev, SANE_FALSE, SANE_TRUE );
-		usbio_WriteReg    ( handle, 0x58, 0x1d );
-		usbio_WriteReg    ( handle, 0x59, 0x49 );
-		usbio_ReadReg     ( handle, 0x02, &t );
-		usbio_WriteReg    ( handle, 0x58, dev->usbDev.HwSetting.bReg_0x58 );
-		usbio_WriteReg    ( handle, 0x59, dev->usbDev.HwSetting.bReg_0x59 );
+		usb_switchLampX( dev, SANE_FALSE, SANE_TRUE );
+		usbio_WriteReg ( handle, 0x58, 0x1d );
+		usbio_WriteReg ( handle, 0x59, 0x49 );
+		usbio_ReadReg  ( handle, 0x02, &t );
+		usbio_WriteReg ( handle, 0x58, dev->usbDev.HwSetting.bReg_0x58 );
+		usbio_WriteReg ( handle, 0x59, dev->usbDev.HwSetting.bReg_0x59 );
 
 		DBG( _DBG_INFO, "REG[0x02] = 0x%02x\n", t );
 
 		if( t & 0x02 ) {
 			DBG( _DBG_INFO, "TPA detected\n" );
-  			dev->usbDev.Caps.wFlags |= DEVCAPSFLAG_TPA;
+			dev->usbDev.Caps.wFlags |= DEVCAPSFLAG_TPA;
 		} else
 			DBG( _DBG_INFO, "TPA NOT detected\n" );
 
 		if( dev->adj.enableTpa ) {
 			DBG( _DBG_INFO, "Enabled TPA for EPSON (override)\n" );
-  			dev->usbDev.Caps.wFlags |= DEVCAPSFLAG_TPA;
-     	}
-    }
+			dev->usbDev.Caps.wFlags |= DEVCAPSFLAG_TPA;
+		}
+	}
 
-	/*
-     * well now we patch the vendor string...
-     * if not found, the default vendor will be Plustek
-     */
+	/* well now we patch the vendor string...
+	 * if not found, the default vendor will be Plustek
+	 */
 	for( i = 0; usbVendors[i].desc != NULL; i++ ) {
 
 		if( usbVendors[i].id == vendor ) {
@@ -204,8 +282,11 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	}
 
 	dev->usbDev.currentLamp = usb_GetLampStatus( dev );
-
 	usb_ResetRegisters( dev );
+
+	if( dev->initialized >= 0 )
+		return;
+	
     usbio_ResetLM983x ( dev );
 	usb_IsScannerReady( dev );
 
@@ -215,13 +296,30 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	sParam.bDataType     = SCANDATATYPE_Color;
 	sParam.bSource       = SOURCE_Reflection;
 	sParam.Origin.x      = 0;
-    sParam.Origin.y      = 0;
+	sParam.Origin.y      = 0;
 	sParam.siThreshold   = 0;
 	sParam.UserDpi.x     = 150;
-    sParam.UserDpi.y     = 150;
+	sParam.UserDpi.y     = 150;
 	sParam.dMCLK         = 4;
 	sParam.Size.dwPixels = 0;
 
+	/* create calibration-filename */
+	ptr = getenv ("HOME");
+    if( !usb_normFileName( dev->usbDev.ModelStr, tmp_str1, PATH_MAX )) {
+		strcpy( tmp_str1, "plustek-default" );
+	}
+	
+    if( NULL == ptr ) {
+		sprintf( tmp_str2, "/tmp/%s-%s.cal",
+				 dev->sane.vendor, tmp_str1 );
+	} else {
+		sprintf( tmp_str2, "%s/.sane/%s-%s.cal",
+				 ptr, dev->sane.vendor, tmp_str1 );
+	}
+	dev->calFile = strdup( tmp_str2 );
+	DBG( _DBG_INFO, "Calibration file-name set to:\n" );
+	DBG( _DBG_INFO, ">%s<\n", dev->calFile );
+
 	/* initialize the ASIC registers */
 	usb_SetScanParameters( dev, &sParam );
 
@@ -229,7 +327,7 @@ static void usb_initDev( pPlustek_Device dev, int idx, int handle, int vendor )
 	usb_ModuleToHome( dev, SANE_FALSE );
 
 	/* set the global flag, that we are initialized so far */
-	dev->initialized = SANE_TRUE;
+	dev->initialized = idx;
 }
 
 /**
@@ -329,14 +427,14 @@ static void usbDev_shutdown( Plustek_Device *dev  )
 	
 		if( 0 != dev->usbDev.bLampOffOnEnd ) {
 
-    		DBG( _DBG_INFO, "Switching lamp off...\n" );
+			DBG( _DBG_INFO, "Switching lamp off...\n" );
 			usb_LampOn( dev, SANE_FALSE, SANE_FALSE );
-		}		
+		}
 		
 		dev->fd = -1;
 		sanei_usb_close( handle );
 	}
-	
+
 	usb_StopLampTimer( dev );
 }
 
@@ -362,7 +460,6 @@ static SANE_Bool usb_IsDeviceInList( char *usbIdStr )
 }
 
 /**
- *
  */
 static SANE_Status usb_attach( SANE_String_Const dev_name )
 {
@@ -404,7 +501,7 @@ static SANE_Bool usbDev_autodetect( SANE_Word *vendor, SANE_Word *product )
 
 			*vendor  = v;
 			*product = p;
-       		return SANE_TRUE;
+			return SANE_TRUE;
 		}
 	}
 
@@ -412,26 +509,25 @@ static SANE_Bool usbDev_autodetect( SANE_Word *vendor, SANE_Word *product )
 }
 
 /**
- *
  */
-static int usbDev_open( const char *dev_name, void *misc )
+static int usbDev_open( Plustek_Device *dev )
 {
-    char            devStr[50];
-    int             result;
-	int             i;
-	SANE_Int        handle;
-	SANE_Byte       version;
-	SANE_Word    	vendor, product;
-	SANE_Bool       was_empty;
-    pPlustek_Device dev = (pPlustek_Device)misc;
+	char      devStr[50];
+	int       result;
+	int       i;
+	int       lc;
+	SANE_Int  handle;
+	SANE_Byte version;
+	SANE_Word vendor, product;
+	SANE_Bool was_empty;
 
-    DBG( _DBG_INFO, "usbDev_open(%s,%s)\n", dev_name, dev->usbId );
+	DBG( _DBG_INFO, "usbDev_open(%s,%s)\n", dev->name, dev->usbId );
 
-    /* preset our internal usb device structure */
-   	memset( &dev->usbDev, 0, sizeof(DeviceDef));
+	/* preset our internal usb device structure */
+	memset( &dev->usbDev, 0, sizeof(DeviceDef));
 	USB_devname[0] = '\0';
 
-	if( !strcmp( dev_name, "auto" )) {
+	if( !strcmp( dev->name, "auto" )) {
 
 		if( dev->usbId[0] == '\0' ) {
 
@@ -465,7 +561,7 @@ static int usbDev_open( const char *dev_name, void *misc )
 
 	} else {
 
-		if( SANE_STATUS_GOOD != sanei_usb_open( dev_name, &handle ))
+		if( SANE_STATUS_GOOD != sanei_usb_open( dev->name, &handle ))
     		return -1;
 	}
 	
@@ -495,7 +591,7 @@ static int usbDev_open( const char *dev_name, void *misc )
 
 	} else {
 
-		DBG( _DBG_INFO, "Can't get vendor ID from driver...\n" );
+		DBG( _DBG_INFO, "Can't get vendor & product ID from driver...\n" );
 
 		/* if the ioctl stuff is not supported by the kernel and we have
          * nothing specified, we have to give up...
@@ -507,44 +603,48 @@ static int usbDev_open( const char *dev_name, void *misc )
 	        return -1;
 		}
 		
-		vendor = strtol( dev->usbId, 0, 0 );
-		DBG( _DBG_INFO, "... using the specified: 0x%04x\n", vendor );
+		vendor  = strtol( &dev->usbId[0], 0, 0 );
+		product = strtol( &dev->usbId[7], 0, 0 );
+		DBG( _DBG_INFO, "... using the specified: "
+		                "0x%04X-0x%04X\n", vendor, product );
 	}
 
-    /*
-     * before accessing the scanner, check if supported!
-     */
-    if( !usb_IsDeviceInList( dev->usbId )) {
-     	DBG( _DBG_ERROR, "Device >%s<, is not supported!\n", dev->usbId );
+	/* before accessing the scanner, check if supported!
+	 */
+	if( !usb_IsDeviceInList( dev->usbId )) {
+		DBG( _DBG_ERROR, "Device >%s<, is not supported!\n", dev->usbId );
 		sanei_usb_close( handle );
 		return -1;
 	}
 
-    if( SANE_STATUS_GOOD != usbio_DetectLM983x( handle, &version )) {
+	if( SANE_STATUS_GOOD != usbio_DetectLM983x( handle, &version )) {
 		sanei_usb_close( handle );
-        return -1;
-    }
+		return -1;
+	}
 
-    if ((version < 3) || (version > 4)) {
+	if ((version < 3) || (version > 4)) {
 		DBG( _DBG_ERROR, "This is not a LM9831 or LM9832 chip based scanner.\n" );
 		sanei_usb_close( handle );
 		return -1;
-    }
+	}
 
 	dev->fd = handle;
-    usbio_ResetLM983x ( dev );
+	usbio_ResetLM983x ( dev );
 	usb_IsScannerReady( dev );
 	dev->fd = -1;
 
 	dev->usbDev.vendor  = vendor;
 	dev->usbDev.product = product;
 
+	DBG( _DBG_INFO, "Detected vendor & product ID: "
+		                "0x%04X-0x%04X\n", vendor, product );
+
 	/*
 	 * Plustek uses the misc IO 1/2 to get the PCB ID
 	 * (PCB = printed circuit board), so it's possible to have one
 	 * product ID and up to 7 different devices...
 	 */
-	if((0x07B3 == vendor) || (0x0458 == vendor)) {
+	if( 0x07B3 == vendor ) {
 	
 		handle = usb_CheckForPlustekDevice( handle, dev );
 	
@@ -554,26 +654,36 @@ static int usbDev_open( const char *dev_name, void *misc )
 		if( handle >= 0 )
 			return handle;	
 		
-	} else {		
+	} else {
 
-	    /* now roam through the setting list... */
-	    strncpy( devStr, dev->usbId, 13 );
-		devStr[13] = '\0';
+		/* now roam through the setting list... */
+		lc = 13;
+		strncpy( devStr, dev->usbId, lc );
+		devStr[lc] = '\0';
+
+		if( 0x400 == vendor ) {
+			if((dev->adj.mov < 0) || (dev->adj.mov > 1)) {
+				DBG(_DBG_INFO, "BearPaw MOV out of range: %d\n", dev->adj.mov);
+				dev->adj.mov = 0;
+			}
+			sprintf( devStr, "%s-%d", dev->usbId, dev->adj.mov );
+			lc = strlen(devStr);
+			DBG( _DBG_INFO, "BearPaw device: %s (%d)\n", devStr, lc );
+		}
 
 		if( was_empty )
 			dev->usbId[0] = '\0';
-		
-	    /*
-    	 * if we don't use the PCD ID extension...
-	     */
+
+		/* if we don't use the PCD ID extension...
+		 */
 		for( i = 0; NULL != Settings[i].pIDString; i++ ) {
 
-			if( 0 == strncmp( Settings[i].pIDString, devStr, 13 )) {
-		    	DBG( _DBG_INFO, "Device description for >%s< found.\n", devStr );
+			if( 0 == strncmp( Settings[i].pIDString, devStr, lc )) {
+				DBG( _DBG_INFO, "Device description for >%s< found.\n", devStr );
 				usb_initDev( dev, i, handle, vendor );
 			    return handle;
 			}
-		}			
+		}
 	}
 
 	sanei_usb_close( handle );
@@ -582,18 +692,16 @@ static int usbDev_open( const char *dev_name, void *misc )
 }
 
 /**
- *
  */
 static int usbDev_close( Plustek_Device *dev )
 {
-    DBG( _DBG_INFO, "usbDev_close()\n" );
+	DBG( _DBG_INFO, "usbDev_close()\n" );
 	sanei_usb_close( dev->fd );
 	dev->fd = -1;
-    return 0;
+	return 0;
 }
 
-/*.............................................................................
- * convert the stuff
+/** convert the stuff
  */
 static int usbDev_getCaps( Plustek_Device *dev )
 {
@@ -601,13 +709,7 @@ static int usbDev_getCaps( Plustek_Device *dev )
 
 	DBG( _DBG_INFO, "usbDev_getCaps()\n" );
 
-	dev->caps.rDataType.wMin    = scaps->Normal.MinDpi.x;
-	dev->caps.rDataType.wDef    = scaps->Normal.MinDpi.x;
-	dev->caps.rDataType.wMax    = scaps->OpticDpi.y;
-	dev->caps.rDataType.wPhyMax = scaps->OpticDpi.x;
-
-	dev->caps.dwBits = _BITS_12;
-	dev->caps.dwFlag = (SFLAG_SCANNERDEV + SFLAG_FLATBED + SFLAG_CUSTOM_GAMMA);
+	dev->caps.dwFlag = 0;
 
 	if(((DEVCAPSFLAG_Positive == scaps->wFlags)  &&
 	    (DEVCAPSFLAG_Negative == scaps->wFlags)) ||
@@ -615,53 +717,9 @@ static int usbDev_getCaps( Plustek_Device *dev )
 		dev->caps.dwFlag |= SFLAG_TPA;
 	}
 
-	dev->caps.wIOBase = 0;
-	dev->caps.wLens   = 1;
-
 	dev->caps.wMaxExtentX = scaps->Normal.Size.x;
 	dev->caps.wMaxExtentY = scaps->Normal.Size.y;
-	dev->caps.AsicID      = _ASIC_IS_USB;
-	dev->caps.Model       = MODEL_OP_USB;
-	dev->caps.Version     = 0;
-
-    return 0;
-}
-
-/** usbDev_getLensInfo
- * set the info for the scan-area.
- */
-static int usbDev_getLensInfo( Plustek_Device *dev, pLensInfo lens )
-{
-	DBG( _DBG_INFO, "usbDev_getLensInfo()\n" );
-
-	lens->wBeginX = 0;
-	lens->wBeginY = 0;
-
-	lens->rExtentX.wMin	   = 150;
-	lens->rExtentX.wDef    =
-	lens->rExtentX.wMax    =
-	lens->rExtentX.wPhyMax = dev->usbDev.Caps.Normal.Size.x;
-
-	lens->rExtentY.wMin    = 150;
-	lens->rExtentY.wDef    =
-	lens->rExtentY.wMax    =
-	lens->rExtentY.wPhyMax = dev->usbDev.Caps.Normal.Size.y;	
-
-    /* set the DPI stuff */
-	lens->rDpiX.wMin 	  = 16;
-	lens->rDpiX.wDef 	  = 50;
-	lens->rDpiX.wMax 	  = (dev->usbDev.Caps.OpticDpi.x *16);
-	lens->rDpiX.wPhyMax   = dev->usbDev.Caps.OpticDpi.x;
-	lens->rDpiY.wMin 	  = 16;
-	lens->rDpiY.wDef 	  = 50;
-	lens->rDpiY.wMax 	  = (dev->usbDev.Caps.OpticDpi.x *16);
-	lens->rDpiY.wPhyMax   = (dev->usbDev.Caps.OpticDpi.x * 2);
-
-	lens->rDpiY.wMin      = dev->usbDev.Caps.Normal.MinDpi.y;
-
-	DBG( _DBG_INFO, "wMAX=%u, WPHYMAX=%u\n", lens->rDpiY.wMax, lens->rDpiY.wPhyMax );
-
-    return 0;
+	return 0;
 }
 
 /** usbDev_getCropInfo
@@ -673,7 +731,7 @@ static int usbDev_getCropInfo( Plustek_Device *dev, pCropInfo ci )
 
 	DBG( _DBG_INFO, "usbDev_getCropInfo()\n" );
 
-    _VAR_NOT_USED(dev);
+	_VAR_NOT_USED(dev);
 
 	usb_GetImageInfo( &ci->ImgDef, &size );
 
@@ -681,7 +739,7 @@ static int usbDev_getCropInfo( Plustek_Device *dev, pCropInfo ci )
 	ci->dwLinesPerArea  = size.dwLines;
 	ci->dwBytesPerLine  = size.dwBytes;
 
-	if( ci->ImgDef.dwFlag & SCANDEF_BoundaryDWORD )
+	if( ci->ImgDef.dwFlag & SCANFLAG_DWORDBoundary )
 		ci->dwBytesPerLine = (ci->dwBytesPerLine + 3UL) & 0xfffffffcUL;
 
 	DBG( _DBG_INFO, "PPL = %lu\n",  ci->dwPixelsPerLine );
@@ -692,7 +750,6 @@ static int usbDev_getCropInfo( Plustek_Device *dev, pCropInfo ci )
 }
 
 /**
- *
  */
 static int usbDev_setMap( Plustek_Device *dev, SANE_Word *map,
 						  SANE_Word length, SANE_Word channel )
@@ -709,38 +766,37 @@ static int usbDev_setMap( Plustek_Device *dev, SANE_Word *map,
 			a_bMap[i]            = (SANE_Byte)map[i];
 			a_bMap[length +i]    = (SANE_Byte)map[i];
 			a_bMap[(length*2)+i] = (SANE_Byte)map[i];
-		}			
-	
+		}
+
 	} else {
-	
+
 		idx = 0;
 		if( channel == _MAP_GREEN )
 			idx = 1;
 		if( channel == _MAP_BLUE )
 			idx = 2;
-			
+
 		for( i = 0; i < length; i++ ) {
 			a_bMap[(length * idx)+i] = (SANE_Byte)map[i];
-		}			
+		}
 	}
 
 	return 0;
 }
 
 /**
- *
  */
 static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 {
-	pHWDef hw = &dev->usbDev.HwSetting;
-
 	DBG( _DBG_INFO, "usbDev_setScanEnv()\n" );
 
     /* clear all the stuff */
     memset( &dev->scanning, 0, sizeof(ScanDef));
 
-	if( si->ImgDef.dwFlag & SCANDEF_Adf && si->ImgDef.dwFlag & SCANDEF_ContinuousScan)
+	if((si->ImgDef.dwFlag & SCANDEF_Adf) &&
+	   (si->ImgDef.dwFlag & SCANDEF_ContinuousScan)) {
 		dev->scanning.sParam.dMCLK = dMCLK_ADF;
+	}
 
     /* Save necessary informations */
 	dev->scanning.fGrayFromColor = 0;
@@ -748,11 +804,24 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 	if( si->ImgDef.wDataType == COLOR_256GRAY ) {
 
 		if( !(si->ImgDef.dwFlag & SCANDEF_Adf ) &&
- 		  (dev->usbDev.Caps.OpticDpi.x == 1200 && si->ImgDef.xyDpi.x <= 300)) {
+		  (dev->usbDev.Caps.OpticDpi.x == 1200 && si->ImgDef.xyDpi.x <= 300)) {
 			dev->scanning.fGrayFromColor = 2;
 			si->ImgDef.wDataType = COLOR_TRUE24;
 
-			DBG( _DBG_INFO, "Gray from color set!\n" );
+			DBG( _DBG_INFO, "* Gray from color set!\n" );
+		}
+
+		if((dev->usbDev.vendor == 0x04A9) && (dev->usbDev.product == 0x2208)) {
+			DBG( _DBG_INFO, "* Gray(GRAY256) from color set (D660U)!\n" );
+			dev->scanning.fGrayFromColor = 2;
+			si->ImgDef.wDataType = COLOR_TRUE24;
+		}
+
+	} else if ( si->ImgDef.wDataType == COLOR_GRAY16 ) {
+		if((dev->usbDev.vendor == 0x04A9) && (dev->usbDev.product == 0x2208)) {
+			DBG( _DBG_INFO, "* Gray(GRAY16) from color set (D660U)!\n" );
+			dev->scanning.fGrayFromColor = 2;
+			si->ImgDef.wDataType = COLOR_TRUE48;
 		}
 	}
 
@@ -765,9 +834,9 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
     								   SCANFLAG_StillModule | SCANDEF_Adf | SCANDEF_ContinuousScan);
 
 	if( !(SCANDEF_QualityScan & si->ImgDef.dwFlag)) {
-		DBG( _DBG_INFO, "Preview Mode set!\n" );
+		DBG( _DBG_INFO, "* Preview Mode set!\n" );
 	} else {
-		DBG( _DBG_INFO, "Preview Mode NOT set!\n" );
+		DBG( _DBG_INFO, "* Preview Mode NOT set!\n" );
 		dev->scanning.dwFlag |= SCANDEF_QualityScan;
 	}
 
@@ -795,23 +864,7 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 	/* on CIS based devices we have to reconfigure the illumination
      * settings for the gray modes
      */
-	if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
-
-		if((dev->scanning.sParam.bDataType == SCANDATATYPE_Gray) ||
-		   (dev->scanning.sParam.bDataType == SCANDATATYPE_BW)) {
-
-			hw->bReg_0x29 = hw->illu_mono.mode;
-
-			memcpy( &hw->red_lamp_on,
-					&hw->illu_mono.red_lamp_on, sizeof(u_short) * 6 );
-
-		} else {
-			hw->bReg_0x29 = hw->illu_color.mode;
-
-			memcpy( &hw->red_lamp_on,
-					&hw->illu_color.red_lamp_on, sizeof(u_short) * 6 );
-		}
-	}
+	usb_AdjustCISLampSettings( dev, SANE_TRUE );
 
 	if( dev->scanning.dwFlag & SCANFLAG_BottomUp)
 		dev->scanning.lBufAdjust = -(long)dev->scanning.dwBytesLine;
@@ -877,11 +930,10 @@ static int usbDev_setScanEnv( Plustek_Device *dev, pScanInfo si )
 }
 
 /**
- *
  */
-static int usbDev_stopScan( Plustek_Device *dev, int *mode )
+static int usbDev_stopScan( Plustek_Device *dev )
 {
-	DBG( _DBG_INFO, "usbDev_stopScan(mode=%u)\n", *mode );
+	DBG( _DBG_INFO, "usbDev_stopScan()\n" );
 
 	/* in cancel-mode we first stop the motor */
 	usb_ScanEnd( dev );
@@ -894,16 +946,14 @@ static int usbDev_stopScan( Plustek_Device *dev, int *mode )
 		dev->scanning.pScanBuffer = NULL;
 		usb_StartLampTimer( dev );
 	}
-
-    return 0;
+	return 0;
 }
 
 /**
- *
  */
-static int usbDev_startScan( Plustek_Device *dev, pStartScan start )
+static int usbDev_startScan( Plustek_Device *dev )
 {
-    pScanDef   scanning = &dev->scanning;
+	pScanDef   scanning = &dev->scanning;
 	static int iSkipLinesForADF = 0;
 
 	DBG( _DBG_INFO, "usbDev_startScan()\n" );
@@ -919,7 +969,7 @@ static int usbDev_startScan( Plustek_Device *dev, pStartScan start )
 
 	/* Allocate shading buffer */
 	if((dev->scanning.dwFlag & SCANDEF_Adf) &&
-	   (dev->scanning.dwFlag & SCANDEF_ContinuousScan)) {
+		(dev->scanning.dwFlag & SCANDEF_ContinuousScan)) {
 		dev->scanning.fCalibrated = SANE_TRUE;
 	} else {
 
@@ -936,17 +986,16 @@ static int usbDev_startScan( Plustek_Device *dev, pStartScan start )
 		scanning->dwFlag |= SCANFLAG_StartScan;
 		usb_LampOn( dev, SANE_TRUE, SANE_TRUE );
 
-		start->dwBytesPerLine = scanning->dwBytesLine;
-		start->dwFlag 		  = scanning->dwFlag;
+		m_fStart    = m_fFirst = SANE_TRUE;
+		m_fAutoPark =
+				(scanning->dwFlag & SCANFLAG_StillModule)?SANE_FALSE:SANE_TRUE;
 		
 		usb_StopLampTimer( dev );
 		return 0;
 	}
-
-    return _E_ALLOC;
+	return _E_ALLOC;
 }
 
-
 /**
  * do the reading stuff here...
  * first we perform the calibration step, and then we read the image
@@ -954,11 +1003,11 @@ static int usbDev_startScan( Plustek_Device *dev, pStartScan start )
  */
 static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
 {
-    int       result;
-    u_long    scaler;
-    pScanDef  scanning = &dev->scanning;
-	pDCapsDef scaps    = &dev->usbDev.Caps;
-	pHWDef    hw       = &dev->usbDev.HwSetting;
+	int       result;
+	SANE_Bool use_alt_cal = SANE_FALSE;
+	pScanDef  scanning    = &dev->scanning;
+	pDCapsDef scaps       = &dev->usbDev.Caps;
+	pHWDef    hw          = &dev->usbDev.HwSetting;
 
 	DBG( _DBG_INFO, "usbDev_PrepareScan()\n" );
 
@@ -967,20 +1016,39 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
 	 */
 	usb_ModuleStatus( dev );
 
-    result = usb_DoCalibration( dev );
-    if( SANE_TRUE != result ) {
-		DBG( _DBG_INFO, "calibration failed!!!\n" );
-        return result;
+	/* the CanoScan CIS devices need special handling... */
+	if((dev->usbDev.vendor == 0x04A9) &&    
+		(dev->usbDev.product==0x2206 || dev->usbDev.product==0x2207 ||
+		 dev->usbDev.product==0x220D || dev->usbDev.product==0x220E)) {
+		use_alt_cal = SANE_TRUE;
+		
+	} else {
+
+		if( dev->adj.altCalibrate ) 
+			use_alt_cal = SANE_TRUE;
 	}
 
+	/* for the skip functionality use the "old" calibration functions */
+	if( dev->usbDev.Caps.workaroundFlag & _WAF_BYPASS_CALIBRATION ) {
+		use_alt_cal = SANE_FALSE;
+	}
+
+	if( use_alt_cal ) {
+		result = cano_DoCalibration( dev );
+	} else {
+		result = usb_DoCalibration( dev );
+	}
+
+	if( SANE_TRUE != result ) {
+		DBG( _DBG_INFO, "calibration failed!!!\n" );
+		return result;
+	}
+
+	if( dev->adj.cacheCalData )
+		usb_SaveCalData( dev );
+
 	DBG( _DBG_INFO, "calibration done.\n" );
 
-	scaler = 1;
-	if((hw->bReg_0x26 & _ONE_CH_COLOR) &&
-	   (scanning->sParam.bDataType == SCANDATATYPE_Color)) {
-		scaler = 3;
-	}
-
 	if( !( scanning->dwFlag & SCANFLAG_Scanning )) {
 
 		usleep( 10 * 1000 );
@@ -990,26 +1058,28 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
 			return 0;
 		}
 
-   		/*
-   		 * if we bypass the calibration step, we wait on lamp warmup here...
-   		 */
-   		if( scaps->workaroundFlag & _WAF_BYPASS_CALIBRATION ) {
-       		if( !usb_Wait4Warmup( dev )) {
-   				DBG( _DBG_INFO, "ReadImage() - Cancel detected...\n" );
-   				return 0;
-   			}
-   		}
+		/*
+		 * if we bypass the calibration step, we wait on lamp warmup here...
+		 */
+		if( scaps->workaroundFlag & _WAF_BYPASS_CALIBRATION ) {
+    		if( !usb_Wait4Warmup( dev )) {
+				DBG( _DBG_INFO, "usbDev_Prepare() - Cancel detected...\n" );
+				return 0;
+			}
+		}
 
-   		scanning->pbScanBufBegin = scanning->pScanBuffer;
+		scanning->pbScanBufBegin = scanning->pScanBuffer;
 
-   		if((dev->caps.dwFlag & SFLAG_ADF) && (scaps->OpticDpi.x == 600))
-   			scanning->dwLinesScanBuf = 8;
-   		else
-   			scanning->dwLinesScanBuf = 32;
+		if((dev->caps.dwFlag & SFLAG_ADF) && (scaps->OpticDpi.x == 600))
+			scanning->dwLinesScanBuf = 8;
+		else
+			scanning->dwLinesScanBuf = 32;
+
+		/* gives faster feedback to the frontend ! */
+		scanning->dwLinesScanBuf = 2;
 
    		scanning->dwBytesScanBuf     = scanning->dwLinesScanBuf *
-   									   scanning->sParam.Size.dwPhyBytes *
-   									   scaler;
+   									   scanning->sParam.Size.dwPhyBytes;
 
    		scanning->dwNumberOfScanBufs = _SCANBUF_SIZE /
    									   scanning->dwBytesScanBuf;
@@ -1017,9 +1087,12 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
    									   scanning->dwLinesScanBuf;
    		scanning->pbScanBufEnd       = scanning->pbScanBufBegin +
    									   scanning->dwLinesPerScanBufs *
-   									   scanning->sParam.Size.dwPhyBytes *
-   									   scaler;
+   									   scanning->sParam.Size.dwPhyBytes;
+		scanning->dwRedShift   = 0;
+		scanning->dwBlueShift  = 0;
+		scanning->dwGreenShift = 0;
 
+		/* CCD scanner */
    		if( scanning->sParam.bChannels == 3 ) {
 
    			scanning->dwLinesDiscard = (u_long)scaps->bSensorDistance *
@@ -1112,14 +1185,14 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
    			scanning->dwLinesDiscard = 0;
    			scanning->Green.pb       = scanning->pbScanBufBegin;
 
-			if(( scanning->sParam.bDataType == SCANDATATYPE_Color ) && 
+   			if(( scanning->sParam.bDataType == SCANDATATYPE_Color ) &&
    			   ( hw->bReg_0x26 & _ONE_CH_COLOR )) {
 
+            	u_long len = scanning->sParam.Size.dwPhyBytes / 3;
+
    				scanning->Red.pb   = scanning->pbScanBufBegin;
-   				scanning->Green.pb = scanning->pbScanBufBegin +
-   									 (scanning->sParam.Size.dwPhyBytes );
-   				scanning->Blue.pb  = scanning->pbScanBufBegin +
-   									 (scanning->sParam.Size.dwPhyBytes )* 2UL;
+   				scanning->Green.pb = scanning->pbScanBufBegin + len;
+   				scanning->Blue.pb  = scanning->pbScanBufBegin + len * 2UL;
        		}
    		}
 
@@ -1153,6 +1226,8 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
 		}
 	}
 
+	dumpPicInit( &scanning->sParam, "plustek-pic.raw" );
+	
 	/* here the NT driver uses an extra reading thread...
      * as the SANE stuff already forked the driver to read data, I think
      * we should only read data by using a function...
@@ -1195,8 +1270,6 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
 
 	scanning->pbGetDataBuf = scanning->pbScanBufBegin;
 
-	dumpPic( "plustek-pic.raw", NULL, 0 );
-
 	scanning->dwLinesToProcess = usb_ReadData( dev  );
 	if( 0 == scanning->dwLinesToProcess )
 		return _E_DATAREAD;
@@ -1205,126 +1278,114 @@ static int usbDev_Prepare( struct Plustek_Device *dev, SANE_Byte *buf )
 }
 
 /**
- *
  */
 static int usbDev_readLine( struct Plustek_Device *dev )
 {
 	int       wrap;
-	u_long    cur, scaler;
-    pScanDef  scanning = &dev->scanning;
+	u_long    cur;
+	pScanDef  scanning = &dev->scanning;
 	pHWDef    hw       = &dev->usbDev.HwSetting;
 
- 	scaler = 1;
-	if((hw->bReg_0x26 & _ONE_CH_COLOR) &&
-	   (scanning->sParam.bDataType == SCANDATATYPE_Color)) {
-		scaler = 3;
-    }
-
 	cur = scanning->dwLinesUser;
 
 	/* we stay within this sample loop until one line has been processed for
-     * the user...
+	 * the user...
 	 */
 	while( cur == scanning->dwLinesUser ) {
 
+		if( usb_IsEscPressed()) {
+			DBG( _DBG_INFO, "readLine() - Cancel detected...\n" );
+			return _E_ABORT;
+		}
 
-        if( usb_IsEscPressed()) {
-        	DBG( _DBG_INFO, "readLine() - Cancel detected...\n" );
-        	return _E_ABORT;
-        }
+		if( !(scanning->dwFlag & SCANFLAG_SampleY))	{
 
-        if( !(scanning->dwFlag & SCANFLAG_SampleY))	{
+			scanning->pfnProcess( dev );
 
-        	scanning->pfnProcess( dev );
+			/* Adjust user buffer pointer */
+			scanning->UserBuf.pb += scanning->lBufAdjust;
+			scanning->dwLinesUser--;
 
-        	/* Adjust user buffer pointer */
-        	scanning->UserBuf.pb += scanning->lBufAdjust;
-        	scanning->dwLinesUser--;
+		} else {
 
-        } else {
+			scanning->wSumY += scanning->sParam.UserDpi.y;
 
-        	scanning->wSumY += scanning->sParam.UserDpi.y;
+			if( scanning->wSumY >= scanning->sParam.PhyDpi.y ) {
+				scanning->wSumY -= scanning->sParam.PhyDpi.y;
 
-        	if( scanning->wSumY >= scanning->sParam.PhyDpi.y ) {
-        		scanning->wSumY -= scanning->sParam.PhyDpi.y;
+				scanning->pfnProcess( dev );
 
-        		scanning->pfnProcess( dev );
+				/* Adjust user buffer pointer */
+				scanning->UserBuf.pb += scanning->lBufAdjust;
+				scanning->dwLinesUser--;
+			}
+		}
 
-        		/* Adjust user buffer pointer */
-        		scanning->UserBuf.pb += scanning->lBufAdjust;
-        		scanning->dwLinesUser--;
-        	}
-        }
-
-        /* Adjust get buffer pointers */
+		/* Adjust get buffer pointers */
 		wrap = 0;
 
-        if( scanning->sParam.bDataType == SCANDATATYPE_Color ) {
+		if( scanning->sParam.bDataType == SCANDATATYPE_Color ) {
 
-        	scanning->Red.pb += (scanning->sParam.Size.dwPhyBytes * scaler);
-        	if( scanning->Red.pb >= scanning->pbScanBufEnd ) {
-        		scanning->Red.pb = scanning->pbScanBufBegin +
-        						   scanning->dwRedShift;
-        		wrap = 1;
-        	}
+			scanning->Red.pb += scanning->sParam.Size.dwPhyBytes;
+			if( scanning->Red.pb >= scanning->pbScanBufEnd ) {
+				scanning->Red.pb = scanning->pbScanBufBegin +
+								   scanning->dwRedShift;
+				wrap = 1;
+			}
 
-        	scanning->Green.pb += (scanning->sParam.Size.dwPhyBytes * scaler);
-        	if( scanning->Green.pb >= scanning->pbScanBufEnd ) {
-        		scanning->Green.pb = scanning->pbScanBufBegin +
-        							 scanning->dwGreenShift;
-        		wrap = 1;
-        	}
+			scanning->Green.pb += scanning->sParam.Size.dwPhyBytes;
+			if( scanning->Green.pb >= scanning->pbScanBufEnd ) {
+				scanning->Green.pb = scanning->pbScanBufBegin +
+									 scanning->dwGreenShift;
+				wrap = 1;
+			}
 
-        	scanning->Blue.pb += (scanning->sParam.Size.dwPhyBytes * scaler);
-        	if( scanning->Blue.pb >= scanning->pbScanBufEnd ) {
-        		scanning->Blue.pb = scanning->pbScanBufBegin +
-        								scanning->dwBlueShift;
-        		wrap = 1;
-        	}
-        } else {
-        	scanning->Green.pb += scanning->sParam.Size.dwPhyBytes;
-        	if( scanning->Green.pb >= scanning->pbScanBufEnd )
-        		scanning->Green.pb = scanning->pbScanBufBegin +
-        							 scanning->dwGreenShift;
-        }
+			scanning->Blue.pb += scanning->sParam.Size.dwPhyBytes;
+			if( scanning->Blue.pb >= scanning->pbScanBufEnd ) {
+				scanning->Blue.pb = scanning->pbScanBufBegin +
+									scanning->dwBlueShift;
+				wrap = 1;
+			}
+		} else {
+			scanning->Green.pb += scanning->sParam.Size.dwPhyBytes;
+			if( scanning->Green.pb >= scanning->pbScanBufEnd )
+				scanning->Green.pb = scanning->pbScanBufBegin +
+									 scanning->dwGreenShift;
+		}
 
-        /*
-         * on any wrap-around of the get pointers in one channel mode
-         * we have to reset them
-         */
-        if( wrap ) {
+		/* on any wrap-around of the get pointers in one channel mode
+		 * we have to reset them
+		 */
+		if( wrap ) {
 
-        	if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
+			u_long len = scanning->sParam.Size.dwPhyBytes;
 
-        		scanning->Red.pb   = scanning->pbScanBufBegin;
-        		scanning->Green.pb = scanning->pbScanBufBegin +
-        								 (scanning->sParam.Size.dwPhyBytes );
-        		scanning->Blue.pb  = scanning->pbScanBufBegin +
-       								 (scanning->sParam.Size.dwPhyBytes )* 2UL;
-        	}
-        }
+			if( hw->bReg_0x26 & _ONE_CH_COLOR ) {
 
-        /*
-         * line processed, check if we have to get more...
-         */
-        scanning->dwLinesToProcess--;
+				if(scanning->sParam.bDataType == SCANDATATYPE_Color) {
+					len /= 3;
+				}
+				scanning->Red.pb   = scanning->pbScanBufBegin;
+				scanning->Green.pb = scanning->pbScanBufBegin + len;
+				scanning->Blue.pb  = scanning->pbScanBufBegin + len * 2UL;
+			}
+		}
 
-        if( 0 == scanning->dwLinesToProcess ) {
+		/* line processed, check if we have to get more...
+		 */
+		scanning->dwLinesToProcess--;
 
-        	scanning->dwLinesToProcess = usb_ReadData( dev );
-        	if( 0 == scanning->dwLinesToProcess ) {
+		if( 0 == scanning->dwLinesToProcess ) {
 
-        		if( usb_IsEscPressed())
-        			return _E_ABORT;
-        	}
-        }
+			scanning->dwLinesToProcess = usb_ReadData( dev );
+			if( 0 == scanning->dwLinesToProcess ) {
+
+				if( usb_IsEscPressed())
+					return _E_ABORT;
+			}
+		}
 	}
-
-	/* after reaching the last line, home the sensor... */
-	if( 0 == scanning->dwLinesUser )
-		usb_ScanEnd( dev );
-
-    return 0; 
+	return 0;
 }
 
 /* END PLUSTEK-USB.C ........................................................*/
