@@ -7,7 +7,7 @@
  *  @brief Main defines for the USB devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -28,6 +28,8 @@
  *        - removed _WAF_FIX_GAIN and _WAF_FIX_OFS
  *        - added skipCoarseCalib to ScanDef
  *        - added additional defines for cis and epson-ccd sensor
+ * - 0.47 - cleanup work
+ *        - added gamma to struct HWDefault
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -220,7 +222,7 @@ enum _DEVCAPSFLAG
 enum _WORKAROUNDS
 {
 	_WAF_NONE               = 0x00000000, /* no fix anywhere needed          */
-	_WAF_BSHIFT7_BUG		= 0x00000001, /* to fix U12 bug in 14bit mode    */
+	_WAF_BSHIFT7_BUG        = 0x00000001, /* to fix U12 bug in 14bit mode    */
 	_WAF_MISC_IO_LAMPS      = 0x00000002, /* special lamp switching          */
 	_WAF_BLACKFINE          = 0x00000004, /* use black calibration strip     */
 	_WAF_BYPASS_CALIBRATION = 0x00000008, /* no calibration,use linear gamma */
@@ -256,14 +258,15 @@ typedef enum
 	MODEL_KaoHsiung = 0,
 	MODEL_HuaLien,
 	MODEL_Tokyo600,
-	MODEL_EPSON,          /**< for EPSON1250/1260         */
-	MODEL_MUSTEK600,      /**< for BearPaw 1200           */
-	MODEL_MUSTEK1200,     /**< for BearPaw 2400           */
-	MODEL_HP,             /**< for HP2x00                 */
-	MODEL_CANON600 ,      /**< for CanoScan 600dpi models */
-	MODEL_CANON1200,      /**< for Canon 1200dpi models   */
-	MODEL_UMAX,           /**< for UMAX 3400/3450         */
-	MODEL_UMAX1200,       /**< for UMAX 5400              */
+	MODEL_EPSON,          /**< for EPSON1250/1260           */
+	MODEL_MUSTEK600,      /**< for BearPaw 1200             */
+	MODEL_MUSTEK1200,     /**< for BearPaw 2400             */
+	MODEL_HP,             /**< for HP2x00                   */
+	MODEL_CANON600 ,      /**< for CanoScan 600dpi models   */
+	MODEL_CANON1200,      /**< for Canon 1200dpi models     */
+	MODEL_CANONCCD1200,   /**< for Canon CCD 1200dpi models */
+	MODEL_UMAX,           /**< for UMAX 3400/3450           */
+	MODEL_UMAX1200,       /**< for UMAX 5400                */
 	MODEL_LAST
 } eModelDef;
 
@@ -367,26 +370,24 @@ typedef struct
  */
 typedef struct HWDefault
 {
-	double				dMaxMotorSpeed;			/* Inches/second, max. scan speed */
-	double				dMaxMoveSpeed;			/* Inches/second, max. move speed */
-	double				dIntegrationTimeLowLamp;
-	double				dIntegrationTimeHighLamp;
-	u_short				wMotorDpi;				/* Full step DPI */
-	u_short				wDRAMSize;				/* in KB         */
-	double				dMinIntegrationTimeLowres;
-												/* in ms.        */
-	double				dMinIntegrationTimeHighres;
-												/* in ms.        */
-	u_short				wGreenPWMDutyCycleLow;
-	u_short				wGreenPWMDutyCycleHigh;
+	double              dMaxMotorSpeed;    /* Inches/second, max. scan speed */
+	double              dMaxMoveSpeed;     /* Inches/second, max. move speed */
+	double              dIntegrationTimeLowLamp;
+	double              dIntegrationTimeHighLamp;
+	u_short             wMotorDpi;                      /* Full step DPI */
+	u_short             wDRAMSize;                      /* in KB         */
+	double              dMinIntegrationTimeLowres;      /*in ms.     */
+	double              dMinIntegrationTimeHighres;     /* in ms.    */
+	u_short             wGreenPWMDutyCycleLow;
+	u_short             wGreenPWMDutyCycleHigh;
 	/* Registers */
-	u_char				bSensorConfiguration;	/* 0x0b */
+	u_char              bSensorConfiguration;	/* 0x0b */
 	/* Sensor control settings */
-	u_char				bReg_0x0c;
-	u_char				bReg_0x0d;
-	u_char				bReg_0x0e;
-	u_char				bReg_0x0f_Mono [10];	/* 0x0f to 0x18 */
-	u_char				bReg_0x0f_Color [10];	/* 0x0f to 0x18 */
+	u_char              bReg_0x0c;
+	u_char              bReg_0x0d;
+	u_char              bReg_0x0e;
+	u_char              bReg_0x0f_Mono [10];	/* 0x0f to 0x18 */
+	u_char              bReg_0x0f_Color [10];	/* 0x0f to 0x18 */
 
 	/* color mode settings */	
 	u_char              bReg_0x26;
@@ -403,46 +404,46 @@ typedef struct HWDefault
 	 * format, you have to pay your attention when you
 	 * write this value to register.
 	 */
-	u_short				StepperPhaseCorrection;	
+	u_short             StepperPhaseCorrection;	
 	
 	/* Sensor Pixel Configuration
 	 * Actually, the wActivePixelsStart will be set to 0 for shading purpose.
      * We have to keep these values to adjust the origins when user does the
      * scan. These settings are based on optic resolution.
      */
-	u_char				bOpticBlackStart;		/* 0x1c        */
-	u_char				bOpticBlackEnd;			/* 0x1d        */
-	u_short				wActivePixelsStart;		/* 0x1e & 0x1f */
-	u_short				wLineEnd;				/* 0x20 & 0x21 */
+	u_char              bOpticBlackStart;       /* 0x1c        */
+	u_char              bOpticBlackEnd;         /* 0x1d        */
+	u_short             wActivePixelsStart;     /* 0x1e & 0x1f */
+	u_short             wLineEnd;               /* 0x20 & 0x21 */
 	
 	/* illumination settings (runtime) */
-	u_short             red_lamp_on;			/* 0x2c & 0x2d */
-	u_short             red_lamp_off;			/* 0x2e & 0x2f */
-	u_short             green_lamp_on;			/* 0x30 & 0x31 */
-	u_short             green_lamp_off;			/* 0x32 & 0x33 */
-	u_short             blue_lamp_on;			/* 0x34 & 0x35 */
-	u_short             blue_lamp_off;			/* 0x36 & 0x37 */
+	u_short             red_lamp_on;            /* 0x2c & 0x2d */
+	u_short             red_lamp_off;           /* 0x2e & 0x2f */
+	u_short             green_lamp_on;          /* 0x30 & 0x31 */
+	u_short             green_lamp_off;         /* 0x32 & 0x33 */
+	u_short             blue_lamp_on;           /* 0x34 & 0x35 */
+	u_short             blue_lamp_off;          /* 0x36 & 0x37 */
 	
 	/* Misc */
-	u_char				bReg_0x45;
-	u_short				wStepsAfterPaperSensor2;/* 0x4c & 0x4d */
+	u_char              bReg_0x45;
+	u_short             wStepsAfterPaperSensor2;/* 0x4c & 0x4d */
 	u_char              bStepsToReverse;        /* 0x50        */
-	u_char				bReg_0x51;
-	u_char				bReg_0x54;
-	u_char				bReg_0x55;
-	u_char				bReg_0x56;
-	u_char				bReg_0x57;
-	u_char				bReg_0x58;
-	u_char				bReg_0x59;
-	u_char				bReg_0x5a;
-	u_char				bReg_0x5b;
-	u_char				bReg_0x5c;
-	u_char				bReg_0x5d;
-	u_char				bReg_0x5e;
+	u_char              bReg_0x51;
+	u_char              bReg_0x54;
+	u_char              bReg_0x55;
+	u_char              bReg_0x56;
+	u_char              bReg_0x57;
+	u_char              bReg_0x58;
+	u_char              bReg_0x59;
+	u_char              bReg_0x5a;
+	u_char              bReg_0x5b;
+	u_char              bReg_0x5c;
+	u_char              bReg_0x5d;
+	u_char              bReg_0x5e;
 	
 	eChipDef            chip;           /* chiptype               */
-    eModelDef			motorModel; 	/* to identify used motor */
-
+	eModelDef           motorModel;     /* to identify used motor */
+	double              gamma;          /* gamma setting          */
 } HWDef, *pHWDef;
 
 /** device description during runtime
